@@ -42,18 +42,21 @@ class BaseAction(object):
 
         try:
             argspec = inspect.getargspec(func)
-        except TypeError:
+        except TypeError: # a callable object, not a function
             argspec = inspect.getargspec(func.__call__)
         # use task meta information as extra_args
-        extra_args = {
+        meta_args = {
             'task': task,
             'targets': task.targets,
             'dependencies': task.file_dep,
             'changed': task.dep_changed,
             }
 
+        extra_args = dict(meta_args)
         # tasks parameter options
         extra_args.update(task.options)
+        if task.pos_arg is not None:
+            extra_args[task.pos_arg] = task.pos_arg_val
         kwargs = kwargs.copy()
 
         for key in six.iterkeys(extra_args):
@@ -63,10 +66,10 @@ class BaseAction(object):
 
                 # it is forbidden to use default values for this arguments
                 # because the user might be unware of this magic.
-                if (argspec.defaults and
+                if (key in meta_args and argspec.defaults and
                     len(argspec.defaults) > (len(argspec.args) - (arg_pos+1))):
-                    msg = ("%s.%s: '%s' argument default value not allowed "
-                           "(reserved by doit)"
+                    msg = ("Task %s, action %s(): The argument '%s' is not "
+                           "allowed  to have a default value (reserved by doit)"
                            % (task.name, func.__name__, key))
                     raise InvalidTask(msg)
 
@@ -95,14 +98,18 @@ class CmdAction(BaseAction):
     @ivar save_out: (str) name used to save output in `values`
     @ivar shell: use shell to execute command
                  see subprocess.Popen `shell` attribute
+    @ivar encoding (str): encoding of the process output
+    @ivar decode_error (str): value for decode() `errors` param
+                              while decoding process output
     @ivar pkwargs: Popen arguments except 'stdout' and 'stderr'
     """
 
     def __init__(self, action, task=None, save_out=None, shell=True,
+                 encoding='utf-8', decode_error='replace',
                  **pkwargs): #pylint: disable=W0231
         for forbidden in ('stdout', 'stderr'):
             if forbidden in pkwargs:
-                msg = "CmdAction can't take param named '{}'."
+                msg = "CmdAction can't take param named '{0}'."
                 raise InvalidTask(msg.format(forbidden))
         self._action = action
         self.task = task
@@ -112,6 +119,8 @@ class CmdAction(BaseAction):
         self.values = {}
         self.save_out = save_out
         self.shell = shell
+        self.encoding = encoding
+        self.decode_error = decode_error
         self.pkwargs = pkwargs
 
     @property
@@ -126,8 +135,8 @@ class CmdAction(BaseAction):
 
 
     def _print_process_output(self, process, input_, capture, realtime):
-        """read 'input_' untill process is terminated
-        write 'input_' content to 'capture' and 'realtime' streams
+        """read 'input_' (bytes) untill process is terminated
+        write 'input_' content to 'capture' (string) and 'realtime' stream
         """
         if realtime:
             if hasattr(realtime, 'encoding'):
@@ -138,7 +147,8 @@ class CmdAction(BaseAction):
         while True:
             # line buffered
             try:
-                line = input_.readline().decode('utf-8')
+                line = input_.readline().decode(self.encoding,
+                                                self.decode_error)
             except:
                 process.terminate()
                 input_.read()
@@ -202,12 +212,12 @@ class CmdAction(BaseAction):
         # it doesnt make so much difference to return as Error or Failed anyway
         if process.returncode > 125:
             return TaskError("Command error: '%s' returned %s" %
-                            (action,process.returncode))
+                             (action, process.returncode))
 
         # task failure
         if process.returncode != 0:
             return TaskFailed("Command failed: '%s' returned %s" %
-                             (action,process.returncode))
+                              (action, process.returncode))
 
         # save stdout in values
         if self.save_out:
@@ -232,6 +242,9 @@ class CmdAction(BaseAction):
             subs_dict['changed'] = " ".join(self.task.dep_changed)
         # task option parameters
         subs_dict.update(self.task.options)
+        # convert postional parameters from list space-separated string
+        if self.task.pos_arg:
+            subs_dict[self.task.pos_arg] = ' '.join(self.task.pos_arg_val)
         return self.action % subs_dict
 
     def __str__(self):
@@ -247,7 +260,18 @@ class Writer(object):
     """write to many streams"""
     def __init__(self, *writers):
         """@param writers - file stream like objects"""
-        self.writers = writers
+        self.writers = []
+        self._isatty = True
+        for writer in writers:
+            self.add_writer(writer)
+
+    def add_writer(self, stream, isatty=None):
+        """adds a stream to the list of writers
+        @param isatty: (bool) if specified overwrites real isatty from stream
+        """
+        self.writers.append(stream)
+        isatty = stream.isatty() if (isatty is None) else isatty
+        self._isatty = self._isatty and isatty
 
     def write(self, text):
         """write 'text' to all streams"""
@@ -259,6 +283,9 @@ class Writer(object):
         for stream in self.writers:
             stream.flush()
 
+    def isatty(self):
+        return self._isatty
+
 
 class PythonAction(BaseAction):
     """Python action. Execute a python callable.
@@ -323,18 +350,20 @@ class PythonAction(BaseAction):
         # set std stream
         old_stdout = sys.stdout
         output = StringIO()
+        out_writer = Writer()
+        # capture output but preserve isatty() from original stream
+        out_writer.add_writer(output, old_stdout.isatty())
+        if out:
+            out_writer.add_writer(out)
+        sys.stdout = out_writer
+
         old_stderr = sys.stderr
         errput = StringIO()
-
-        out_list = [output]
-        if out:
-            out_list.append(out)
-        err_list = [errput]
+        err_writer = Writer()
+        err_writer.add_writer(errput, old_stderr.isatty())
         if err:
-            err_list.append(err)
-
-        sys.stdout = Writer(*out_list)
-        sys.stderr = Writer(*err_list)
+            err_writer.add_writer(err)
+        sys.stderr = err_writer
 
         kwargs = self._prepare_kwargs()
 
