@@ -1,12 +1,29 @@
 import inspect
 import sys
+from collections import deque
 
+from . import version
 from .cmdparse import CmdOption, CmdParse
-from .exceptions import InvalidCommand
+from .exceptions import InvalidCommand, InvalidDodoFile
 from .dependency import backend_map
 from . import loader
 
 
+def version_tuple(ver_in):
+    """convert a version string or tuple into a 3-element tuple with ints
+    Any part that is not a number (dev0, a2, b4) will be converted to -1
+    """
+    result = []
+    parts = ver_in.split('.') if isinstance(ver_in, str) else ver_in
+    for rev in parts:
+        try:
+            result.append(int(rev))
+        except:
+            result.append(-1)
+    assert len(result) == 3
+    return result
+
+
 class Command(object):
     """third-party should subclass this for commands that do no use tasks
 
@@ -30,10 +47,16 @@ class Command(object):
     # sequence of dicts
     cmd_options = tuple()
 
+    # `execute_tasks` indicates wheather this command execute task's actions.
+    # This is used by the loader to indicate when delayed task creation
+    # should be used.
+    execute_tasks = False
+
     def __init__(self):
         self.name = self.name or self.__class__.__name__.lower()
         Command.CMD_LIST.append(self.name)
         self.options = self.set_options()
+        self.opt_values = None # option values
 
     def set_options(self):
         """@reutrn list of CmdOption
@@ -57,6 +80,7 @@ class Command(object):
         @returns: result of self.execute
         """
         params, args = CmdParse(self.options).parse(in_args)
+        self.opt_values = params
         return self.execute(params, args)
 
 
@@ -81,13 +105,14 @@ class Command(object):
 ######################################################################
 
 # choose internal dependency file.
-opt_depfile = {'name': 'dep_file',
-               'short':'',
-               'long': 'db-file',
-               'type': str,
-               'default': ".doit.db",
-               'help': "file used to save successful runs"
-               }
+opt_depfile = {
+    'name': 'dep_file',
+    'short':'',
+    'long': 'db-file',
+    'type': str,
+    'default': ".doit.db",
+    'help': "file used to save successful runs"
+}
 
 # dependency file DB backend
 opt_backend = {
@@ -96,40 +121,43 @@ opt_backend = {
     'long': 'backend',
     'type': str,
     'default': "dbm",
-    'help': ("Select dependency file backend." +
+    'help': ("Select dependency file backend. " +
              "Available options dbm, json, sqlite3. [default: %(default)s]")
 }
 
 
 #### options related to dodo.py
 # select dodo file containing tasks
-opt_dodo = {'name': 'dodoFile',
-            'short':'f',
-            'long': 'file',
-            'type': str,
-            'default': 'dodo.py',
-            'help':"load task from dodo FILE [default: %(default)s]"
-            }
+opt_dodo = {
+    'name': 'dodoFile',
+    'short':'f',
+    'long': 'file',
+    'type': str,
+    'default': 'dodo.py',
+    'help':"load task from dodo FILE [default: %(default)s]"
+}
 
 # cwd
-opt_cwd = {'name': 'cwdPath',
-           'short':'d',
-           'long': 'dir',
-           'type': str,
-           'default': None,
-           'help':("set path to be used as cwd directory (file paths on " +
-                   "dodo file are relative to dodo.py location.")
-           }
+opt_cwd = {
+    'name': 'cwdPath',
+    'short':'d',
+    'long': 'dir',
+    'type': str,
+    'default': None,
+    'help':("set path to be used as cwd directory (file paths on " +
+            "dodo file are relative to dodo.py location).")
+}
 
 # seek dodo file on parent folders
-opt_seek_file = {'name': 'seek_file',
-                 'short': 'k',
-                 'long': 'seek-file',
-                 'type': bool,
-                 'default': False,
-                 'help': ("seek dodo file on parent folders " +
-                          "[default: %(default)s]")
-                 }
+opt_seek_file = {
+    'name': 'seek_file',
+    'short': 'k',
+    'long': 'seek-file',
+    'type': bool,
+    'default': False,
+    'help': ("seek dodo file on parent folders " +
+             "[default: %(default)s]")
+}
 
 
 class TaskLoader(object):
@@ -153,13 +181,13 @@ class TaskLoader(object):
         raise NotImplementedError()
 
     @staticmethod
-    def _load_from_module(module, cmd_list):
+    def _load_from(cmd, namespace, cmd_list):
         """load task from a module or dict with module members"""
-        if inspect.ismodule(module):
-            members = dict(inspect.getmembers(module))
+        if inspect.ismodule(namespace):
+            members = dict(inspect.getmembers(namespace))
         else:
-            members = module
-        task_list = loader.load_tasks(members, cmd_list)
+            members = namespace
+        task_list = loader.load_tasks(members, cmd_list, cmd.execute_tasks)
         doit_config = loader.load_doit_config(members)
         return task_list, doit_config
 
@@ -174,7 +202,7 @@ class ModuleTaskLoader(TaskLoader):
         self.mod_dict = mod_dict
 
     def load_tasks(self, cmd, params, args):
-        return self._load_from_module(self.mod_dict, cmd.CMD_LIST)
+        return self._load_from(cmd, self.mod_dict, cmd.CMD_LIST)
 
 
 class DodoTaskLoader(TaskLoader):
@@ -184,7 +212,7 @@ class DodoTaskLoader(TaskLoader):
     def load_tasks(self, cmd, params, args):
         dodo_module = loader.get_module(params['dodoFile'], params['cwdPath'],
                                         params['seek_file'])
-        return self._load_from_module(dodo_module, cmd.CMD_LIST)
+        return self._load_from(cmd, dodo_module, cmd.CMD_LIST)
 
 
 
@@ -199,6 +227,7 @@ class DoitCmdBase(Command):
     def __init__(self, task_loader=None, dep_file=None, backend=None,
                  config=None, task_list=None, sel_tasks=None, outstream=None):
         """this initializer is usually just used on tests"""
+        # FIXME 'or TaskLoader()' below is hack for tests
         self._loader = task_loader or TaskLoader()
         Command.__init__(self)
         # TODO: helper to test code should not be here
@@ -226,6 +255,16 @@ class DoitCmdBase(Command):
                                                               args)
         self.sel_tasks = args or self.config.get('default_tasks')
 
+        # check minversion
+        minversion = self.config.get('minversion')
+        if minversion:
+            if version_tuple(minversion) > version_tuple(version.VERSION):
+                msg = ('Please update doit. '
+                'Minimum version required is {required}. '
+                'You are using {actual}. ')
+                raise InvalidDodoFile(msg.format(required=minversion,
+                                                 actual=version.VERSION))
+
         # merge config values into params
         params.update_defaults(self.config)
         self.dep_file = params['dep_file']
@@ -235,7 +274,7 @@ class DoitCmdBase(Command):
 
         # magic - create dict based on signature of _execute method
         args_name = inspect.getargspec(self._execute)[0]
-        exec_params = dict((n, params[n]) for n in args_name if n!='self')
+        exec_params = dict((n, params[n]) for n in args_name if n != 'self')
         return self._execute(**exec_params)
 
 
@@ -259,16 +298,16 @@ def tasks_and_deps_iter(tasks, sel_tasks, yield_duplicates=False):
     @param sel_tasks(list - str)
     """
     processed = set() # str - task name
-    to_process = set(sel_tasks) # str - task name
+    to_process = deque(sel_tasks) # str - task name
     # get initial task
     while to_process:
-        task = tasks[to_process.pop()]
+        task = tasks[to_process.popleft()]
         processed.add(task.name)
         yield task
         # FIXME this does not take calc_dep into account
         for task_dep in task.task_dep + task.setup_tasks:
             if (task_dep not in processed) and (task_dep not in to_process):
-                to_process.add(task_dep)
+                to_process.append(task_dep)
             elif yield_duplicates:
                 yield tasks[task_dep]
 
