@@ -31,6 +31,10 @@ task_dep:
   - type: list. items:
     * task name (string)
 
+setup:
+ - type: list. items:
+   * task name (string)
+
 targets:
   - type: list of strings
   - each item is file-path relative to the dodo file (accepts both files and folders)
@@ -47,13 +51,9 @@ calc_dep:
 
 getargs:
   - type: dictionary
-    * key: string with the name of the function parater (used in a python-action)
+    * key: string with the name of the function argument (used in a python-action)
     * value: tuple of (<task-name>, <variable-name>)
 
-setup:
- - type: list. items:
-   * task name (string)
-
 teardown:
  - type: (list) of actions (see above)
 
@@ -70,7 +70,12 @@ params:
    - default [required] default value for parameter
    - short [optional] (string - 1 letter) short option string
    - long [optional] (string) long option string
-   - type [optional] the option will be converted to this type
+   - type [optional] (callable) the option will be converted to this type
+   - help [optional] (string) description displayed by help command
+   - inverse [optional] (string) for a bool parameter set value to False
+
+pos_arg:
+ - type: string -> name of the function argument to receive list of positional arguments
 
 verbosity:
  - type: int
@@ -90,7 +95,7 @@ watch:
 
 class Help(DoitCmdBase):
     doc_purpose = "show help"
-    doc_usage = ""
+    doc_usage = "[TASK] [COMMAND]"
     doc_description = None
 
     @staticmethod
