@@ -4,8 +4,8 @@ from collections import deque
 import six
 
 from .exceptions import InvalidTask, InvalidCommand, InvalidDodoFile
-from .task import Task
-
+from .task import Task, DelayedLoaded
+from .loader import generate_tasks
 
 
 class TaskControl(object):
@@ -57,7 +57,7 @@ class TaskControl(object):
                 task.task_dep.extend(self._get_wild_tasks(pattern))
 
         self._check_dep_names()
-        self._init_implicit_deps()
+        self.set_implicit_deps(self.targets, task_list)
 
 
     def _check_dep_names(self):
@@ -75,27 +75,39 @@ class TaskControl(object):
                     raise InvalidTask(msg % (task.name, setup_task))
 
 
-    def _init_implicit_deps(self):
-        """get task_dep based on file_dep on a target from another task"""
+    @staticmethod
+    def set_implicit_deps(targets, task_list):
+        """set/add task_dep based on file_dep on a target from another task
+        @param targets: (dict) fileName -> task_name
+        @param task_list: (list - Task) task with newly added file_dep
+        """
         # 1) create a dictionary associating every target->task. where the task
         # builds that target.
-        for task in six.itervalues(self.tasks):
+        for task in task_list:
             for target in task.targets:
-                if target in self.targets:
+                if target in targets:
                     msg = ("Two different tasks can't have a common target." +
                            "'%s' is a target for %s and %s.")
                     raise InvalidTask(msg % (target, task.name,
-                                             self.targets[target]))
-                self.targets[target] = task.name
+                                             targets[target]))
+                targets[target] = task.name
+
         # 2) now go through all dependencies and check if they are target from
         # another task.
-        for task in six.itervalues(self.tasks):
-            self.add_implicit_task_dep(self.targets, task, task.file_dep)
+        # FIXME - when used with delayed tasks needs to check if
+        #         any new target matches any old file_dep.
+        for task in task_list:
+            TaskControl.add_implicit_task_dep(targets, task, task.file_dep)
 
 
     @staticmethod
     def add_implicit_task_dep(targets, task, deps_list):
-        """add tasks which created targets are file_dep for this task"""
+        """add implicit task_dep for `task` for newly added `file_dep`
+
+        @param targets: (dict) fileName -> task_name
+        @param task: (Task) task with newly added file_dep
+        @param dep_list: (list - str): list of file_dep for task
+        """
         for dep in deps_list:
             if (dep in targets and targets[dep] not in task.task_dep):
                 task.task_dep.append(targets[dep])
@@ -113,16 +125,26 @@ class TaskControl(object):
     def _process_filter(self, task_selection):
         """process cmd line task options
         [task_name [-task_opt [opt_value]] ...] ...
+
+        @param task_selection: list of strings with task names/params or target
+        @return list of task names. Expanding glob and removed params
         """
         filter_list = []
         def add_filtered_task(seq, f_name):
-            """can be filter by target or task name """
+            """add task to list `filter_list` and set task.options from params
+            @return list - str: of elements not yet
+            """
             filter_list.append(f_name)
+            # only tasks specified by name can contain parameters
             if f_name in self.tasks:
                 # parse task_selection
                 the_task = self.tasks[f_name]
                 # remaining items are other tasks not positional options
                 the_task.options, seq = the_task.taskcmd.parse(seq)
+                # if task takes positional parameters set all as pos_arg_val
+                if the_task.pos_arg is not None:
+                    the_task.pos_arg_val = seq
+                    seq = []
             return seq
 
         # process...
@@ -142,7 +164,7 @@ class TaskControl(object):
     def _filter_tasks(self, task_selection):
         """Select tasks specified by filter.
 
-        filter can specify tasks to be execute by task name or target.
+        @param task_selection: list of strings with task names/params or target
         @return (list) of string. where elements are task name.
         """
         selected_task = []
@@ -164,7 +186,10 @@ class TaskControl(object):
 
 
     def process(self, task_selection):
-        """@return (list - string) each element is the name of a task"""
+        """
+        @param task_selection: list of strings with task names/params
+        @return (list - string) each element is the name of a task
+        """
         # execute only tasks in the filter in the order specified by filter
         if task_selection is not None:
             self.selected_tasks = self._filter_tasks(task_selection)
@@ -229,6 +254,14 @@ class ExecNode(object):
         # generator from TaskDispatcher._add_task
         self.generator = None
 
+    def reset_task(self, task, generator):
+        """reset task & generator after task is created by its own `loader`"""
+        task.loader = DelayedLoaded
+        self.task = task
+        self.task_dep = task.task_dep[:]
+        self.calc_dep = task.calc_dep.copy()
+        self.generator = generator
+
     def parent_status(self, parent_node):
         if parent_node.run_status == 'failure':
             self.bad_deps.append(parent_node)
@@ -298,14 +331,21 @@ class TaskDispatcher(object):
         @param task_list (list - str) tasks that node should wait for
         @param calc (bool) task_list is for calc_dep
         """
-        # remove tasks that were already executed from task_list
+        # wait_for: contains tasks that `node` needs to wait for and
+        # were not executed yet.
         wait_for = set()
         for name in task_list:
             dep_node = self.nodes[name]
             if (not dep_node) or dep_node.run_status in (None, 'run'):
                 wait_for.add(name)
             else:
+                # if dep task was already executed:
+                # a) set parent status
                 node.parent_status(dep_node)
+                # b) update dependencies from calc_dep results
+                if calc:
+                    self._process_calc_dep_results(dep_node, node)
+
 
         # update ExecNode setting parent/dependent relationship
         for name in wait_for:
@@ -334,22 +374,41 @@ class TaskDispatcher(object):
         # add calc_dep & task_dep until all processed
         # calc_dep may add more deps so need to loop until nothing left
         while True:
-            for calc_dep in node.calc_dep:
-                yield self._gen_node(node, calc_dep)
-            self._node_add_wait_run(node, node.calc_dep, calc=True)
+            calc_dep_list = list(node.calc_dep)
             node.calc_dep.clear()
-
-            # add task_dep
-            for task_dep in node.task_dep:
-                yield self._gen_node(node, task_dep)
-            self._node_add_wait_run(node, node.task_dep)
+            task_dep_list = node.task_dep[:]
             node.task_dep = []
 
-            if (node.wait_run or node.wait_run_calc):
+            for calc_dep in calc_dep_list:
+                yield self._gen_node(node, calc_dep)
+            self._node_add_wait_run(node, calc_dep_list, calc=True)
+
+            # add task_dep
+            for task_dep in task_dep_list:
+                yield self._gen_node(node, task_dep)
+            self._node_add_wait_run(node, task_dep_list)
+
+            # do not wait until all possible task_dep are created
+            if (node.calc_dep or node.task_dep):
+                continue # pragma: no cover # coverage cant catch this #198
+            elif (node.wait_run or node.wait_run_calc):
                 yield 'wait'
             else:
                 break
 
+        if this_task.loader:
+            ref = this_task.loader.creator
+            new_tasks = generate_tasks(this_task.name, ref(), ref.__doc__)
+            TaskControl.set_implicit_deps(self.targets, new_tasks)
+            for nt in new_tasks:
+                if not nt.loader:
+                    nt.loader = DelayedLoaded
+                self.tasks[nt.name] = nt
+            # this task was placeholder to execute the loader
+            # now it needs to be re-processed with the real task
+            yield "reset generator"
+            assert False, "This generator can not be used again"
+
         # add itself
         yield this_task
 
@@ -427,21 +486,8 @@ class TaskDispatcher(object):
                 # calc_dep might add new deps that can be run without
                 # waiting for the completion of the remaining deps
                 is_ready = True
+                self._process_calc_dep_results(node, waiting_node)
 
-                # refresh this task dependencies with values got from calc_dep
-                values = node.task.values
-                len_task_deps = len(node.task.task_dep)
-                old_calc_dep = node.task.calc_dep.copy()
-                waiting_node.task.update_deps(values)
-                TaskControl.add_implicit_task_dep(
-                    self.targets, waiting_node.task,
-                    values.get('file_dep', []))
-
-                # update node's list of non-processed dependencies
-                new_task_dep = waiting_node.task.task_dep[len_task_deps:]
-                waiting_node.task_dep.extend(new_task_dep)
-                new_calc_dep = waiting_node.task.calc_dep - old_calc_dep
-                waiting_node.calc_dep.update(new_calc_dep)
 
             # this node can be further processed
             if is_ready and (waiting_node in self.waiting):
@@ -449,6 +495,24 @@ class TaskDispatcher(object):
                 self.waiting.remove(waiting_node)
 
 
+    def _process_calc_dep_results(self, node, waiting_node):
+        # refresh this task dependencies with values got from calc_dep
+        values = node.task.values
+        len_task_deps = len(waiting_node.task.task_dep)
+        old_calc_dep = waiting_node.task.calc_dep.copy()
+        waiting_node.task.update_deps(values)
+        TaskControl.add_implicit_task_dep(
+            self.targets, waiting_node.task,
+            values.get('file_dep', []))
+
+        # update node's list of non-processed dependencies
+        new_task_dep = waiting_node.task.task_dep[len_task_deps:]
+        waiting_node.task_dep.extend(new_task_dep)
+        new_calc_dep = waiting_node.task.calc_dep - old_calc_dep
+        waiting_node.calc_dep.update(new_calc_dep)
+
+
+
     def _dispatcher_generator(self, selected_tasks):
         """return generator dispatching tasks"""
         # each selected task will create a tree (from dependencies) of
@@ -486,6 +550,11 @@ class TaskDispatcher(object):
             elif isinstance(next_step, ExecNode):
                 self.ready.append(next_step)
 
+            # node just performed a delayed creation of tasks, restart
+            elif next_step == "reset generator":
+                node.reset_task(self.tasks[node.task.name],
+                                self._add_task(node))
+
             # got 'wait', add ExecNode to waiting queue
             else:
                 assert next_step == "wait"
