@@ -5,17 +5,17 @@ from multiprocessing import Process, Queue as MQueue
 from threading import Thread
 import six
 from six.moves import queue, xrange
+import pickle
 
 from .exceptions import InvalidTask, CatchedException
 from .exceptions import TaskFailed, SetupError, DependencyError, UnmetDependency
-from .control import ExecNode
+from .task import DelayedLoaded
 
 # execution result.
 SUCCESS = 0
 FAILURE = 1
 ERROR = 2
 
-
 class Runner(object):
     """Task runner
 
@@ -31,7 +31,7 @@ class Runner(object):
     def __init__(self, dep_class, dependency_file, reporter, continue_=False,
                  always_execute=False, verbosity=0):
         """@param dependency_file: (string) file path of the db file
-        @param reporter: reporter to be used. It can be a class or an object
+        @param reporter: reporter object to be used
         @param continue_: (bool) execute all tasks even after a task failure
         @param always_execute: (bool) execute even if up-to-date or ignored
         @param verbosity: (int) 0,1,2 see Task.execute
@@ -232,18 +232,48 @@ class Runner(object):
         @ivar task_dispatcher (TaskDispatcher)
         """
         try:
+            if hasattr(self.reporter, 'initialize'):
+                self.reporter.initialize(task_dispatcher.tasks)
             self.run_tasks(task_dispatcher)
         except InvalidTask as exception:
             self.reporter.runtime_error(str(exception))
+            self.final_result = ERROR
         finally:
             self.finish()
         return self.final_result
 
 
-class Hold(object):
-    """Sentinel class: No task ready to be executed"""
-    pass
 
+# JobXXX objects send from main process to sub-process for execution
+class JobHold(object):
+    """Indicates there is no task ready to be executed"""
+    type = object()
+
+class JobTask(object):
+    """Contains a Task object"""
+    type = object()
+    def __init__(self, task):
+        self.name = task.name
+        try:
+            self.task_pickle = pickle.dumps(task)
+        except pickle.PicklingError as excp:
+            msg = """Error on Task: `{}`.
+Task created at execution time that has an attribute than can not be pickled,
+so not feasible to be used with multi-processing. To fix this issue make sure
+the task is pickable or just do not use multi-processing execution.
+
+Original exception {}: {}
+"""
+            raise InvalidTask(msg.format(self.name, excp.__class__, excp))
+
+class JobTaskPickle(object):
+    """dict of Task object excluding attributes that might be unpicklable"""
+    type = object()
+    def __init__(self, task):
+        self.task_dict = task.pickle_safe_dict() # actually a dict to be pickled
+    @property
+    def name(self):
+        return self.task_dict['name']
 
 
 class MReporter(object):
@@ -301,13 +331,12 @@ class MRunner(Runner):
         self.tasks = None    # dict of task instances by name
         self.result_q = None
 
-    def get_next_task(self, completed):
+    def get_next_job(self, completed):
         """get next task to be dispatched to sub-process
 
         On MP needs to check if the dependencies finished its execution
-        @returns: - a ExecNode
-                  - None -> no more tasks to be executed
-                  - Hold object, all tasks are waiting for dependencies
+        @returns : - None -> no more tasks to be executed
+                   - JobXXX
         """
         if self._stop_running:
             return None # gentle stop
@@ -316,16 +345,23 @@ class MRunner(Runner):
             # get next task from controller
             try:
                 node = self.task_dispatcher.generator.send(node)
-                if not isinstance(node, ExecNode):
+                if node == "hold on":
                     self.free_proc += 1
-                    return Hold()
+                    return JobHold()
             # no more tasks from controller...
             except StopIteration:
                 # ... terminate one sub process if no other task waiting
                 return None
 
+            # send a task to be executed
             if self.select_task(node, self.tasks):
-                return node
+                # If sub-process already contains the Task object send
+                # only safe pickle data, otherwise send whole object.
+                task = node.task
+                if task.loader is DelayedLoaded and self.Child == Process:
+                    return JobTask(task)
+                else:
+                    return JobTaskPickle(task)
 
 
     def _run_tasks_init(self, task_dispatcher):
@@ -334,28 +370,39 @@ class MRunner(Runner):
         self.tasks = task_dispatcher.tasks
 
 
-    def _run_start_processes(self, task_q, result_q):
+    def _run_start_processes(self, job_q, result_q):
         """create and start sub-processes
-        @param task_q: (multiprocessing.Queue) tasks to be executed
+        @param job_q: (multiprocessing.Queue) tasks to be executed
         @param result_q: (multiprocessing.Queue) collect task results
         @return list of Process
         """
         proc_list = []
         for _ in xrange(self.num_process):
-            next_node = self.get_next_task(None)
-            if next_node is None:
+            next_job = self.get_next_job(None)
+            if next_job is None:
                 break # do not start more processes than tasks
-            if isinstance(next_node, ExecNode):
-                task_q.put(next_node.task)
-            else:
-                task_q.put(next_node)
+            job_q.put(next_job)
             process = self.Child(
                 target=self.execute_task_subprocess,
-                args=(task_q, result_q))
+                args=(job_q, result_q))
             process.start()
             proc_list.append(process)
         return proc_list
 
+    def _process_result(self, node, task, result):
+        """process result received from sub-process"""
+        if 'failure' in result:
+            catched_excp = result['failure']
+        else:
+            # success set values taken from subprocess result
+            catched_excp = None
+            task.update_from_pickle(result['task'])
+            for action, output in zip(task.actions, result['out']):
+                action.out = output
+            for action, output in zip(task.actions, result['err']):
+                action.err = output
+        self.process_task_result(node, catched_excp)
+
 
     def run_tasks(self, task_dispatcher):
         """controls subprocesses task dispatching and result collection
@@ -363,56 +410,44 @@ class MRunner(Runner):
         # result queue - result collected from sub-processes
         result_q = self.Queue()
         # task queue - tasks ready to be dispatched to sub-processes
-        task_q = self.Queue()
+        job_q = self.Queue()
         self._run_tasks_init(task_dispatcher)
-        proc_list = self._run_start_processes(task_q, result_q)
+        proc_list = self._run_start_processes(job_q, result_q)
 
         # wait for all processes terminate
         proc_count = len(proc_list)
-        while proc_count:
-            # wait until there is a result to be consumed
-            result = result_q.get()
+        try:
+            while proc_count:
+                # wait until there is a result to be consumed
+                result = result_q.get()
 
-            node = task_dispatcher.nodes[result['name']]
-            task = node.task
-            if 'reporter' in result:
-                getattr(self.reporter, result['reporter'])(task)
-                continue
-            elif 'failure' in result:
-                catched_excp = result['failure']
-            elif 'exit' in result:
-                raise result['exit'](result['exception'])
-            else:
-                # success set values taken from subprocess result
-                catched_excp = None
-                task.result = result['result']
-                task.values = result['values']
-                for action, output in zip(task.actions, result['out']):
-                    action.out = output
-                for action, output in zip(task.actions, result['err']):
-                    action.err = output
-
-            # completed one task, dispatch next one
-            self.process_task_result(node, catched_excp)
-
-            # update num free process
-            free_proc = self.free_proc + 1
-            self.free_proc = 0
-            # tries to get as many tasks as free process
-            completed = node
-            for _ in range(free_proc):
-                next_node = self.get_next_task(completed)
-                completed = None
-                if next_node is None:
-                    proc_count -= 1
-                if isinstance(next_node, ExecNode):
-                    task_q.put(next_node.task)
-                else:
-                    task_q.put(next_node)
-
-            # check for cyclic dependencies
-            assert len(proc_list) > self.free_proc
+                if 'exit' in result:
+                    raise result['exit'](result['exception'])
+                node = task_dispatcher.nodes[result['name']]
+                task = node.task
+                if 'reporter' in result:
+                    getattr(self.reporter, result['reporter'])(task)
+                    continue
+                self._process_result(node, task, result)
 
+                # update num free process
+                free_proc = self.free_proc + 1
+                self.free_proc = 0
+                # tries to get as many tasks as free process
+                completed = node
+                for _ in range(free_proc):
+                    next_job = self.get_next_job(completed)
+                    completed = None
+                    if next_job is None:
+                        proc_count -= 1
+                    job_q.put(next_job)
+                # check for cyclic dependencies
+                assert len(proc_list) > self.free_proc
+        except Exception:
+            if self.Child == Process:
+                for proc in proc_list:
+                    proc.terminate()
+            raise
         # we are done, join all process
         for proc in proc_list:
             proc.join()
@@ -425,44 +460,48 @@ class MRunner(Runner):
             getattr(self.reporter, result['reporter'])(task)
 
 
-    def execute_task_subprocess(self, task_q, result_q):
+    def execute_task_subprocess(self, job_q, result_q):
         """executed on child processes
-        @param task_q: task queue,
-            TODO: improve this creating an common interface for stuff
-                  that can be put in this queue
+        @param job_q: task queue,
             * None elements indicate process can terminate
-            * Hold indicate process should wait for next task
-            * Task task to be executed
+            * JobHold indicate process should wait for next task
+            * JobTask / JobTaskPickle task to be executed
         """
         self.result_q = result_q
         if self.Child == Process:
             self.reporter = MReporter(self, self.reporter)
         try:
             while True:
-                recv_task = task_q.get()
-                if recv_task is None:
+                job = job_q.get()
+
+                if job is None:
                     self.teardown()
                     return # no more tasks to execute finish this process
 
-                # do nothing. this used to start the subprocess even if no task
-                # is available when process is created.
-                if isinstance(recv_task, Hold):
-                    continue
-
-                # recv_task is an incomplete obj. when pickled, attrbiutes
+                # job is an incomplete Task obj when pickled, attrbiutes
                 # that might contain unpickleble data were removed.
                 # so we need to get task from this process and update it
                 # to get dynamic task attributes.
-                task = self.tasks[recv_task.name]
-                if self.Child == Process:
-                    task.update_from_pickle(recv_task)
+                if job.type is JobTaskPickle.type:
+                    task = self.tasks[job.name]
+                    if self.Child == Process: # pragma: no cover ...
+                        # ... actually covered but subprocess doesnt get it.
+                        task.update_from_pickle(job.task_dict)
+
+                elif job.type is JobTask.type:
+                    task = pickle.loads(job.task_pickle)
+
+                # do nothing. this is used to start the subprocess even
+                # if no task is available when process is created.
+                else:
+                    assert job.type is JobHold.type
+                    continue # pragma: no cover
 
                 result = {'name': task.name}
                 t_result = self.execute_task(task)
 
                 if t_result is None:
-                    result['result'] = task.result
-                    result['values'] = task.values
+                    result['task'] = task.pickle_safe_dict()
                     result['out'] = [a.out for a in task.actions]
                     result['err'] = [a.err for a in task.actions]
                 else:
@@ -470,15 +509,22 @@ class MRunner(Runner):
                 result_q.put(result)
         except (SystemExit, KeyboardInterrupt, Exception) as exception:
             # error, blow-up everything. send exception info to master process
-            result_q.put({'name': task.name,
-                          'exit': exception.__class__,
-                          'exception': str(exception)})
+            result_q.put({
+                'exit': exception.__class__,
+                'exception': str(exception)})
 
 
 class MThreadRunner(MRunner):
     """Parallel runner using threads"""
     Queue = staticmethod(queue.Queue)
-    Child = staticmethod(Thread)
+    class DaemonThread(Thread):
+        """daemon thread to make sure process is terminated if there is
+        an uncatch exception and threads are not correctly joined.
+        """
+        def __init__(self, *args, **kwargs):
+            Thread.__init__(self, *args, **kwargs)
+            self.daemon = True
+    Child = staticmethod(DaemonThread)
 
     @staticmethod
     def available():
