@@ -10,6 +10,7 @@ import six
 from .cmdparse import CmdOption, TaskParse
 from .exceptions import CatchedException, InvalidTask
 from .action import create_action, PythonAction
+from .dependency import UptodateCalculator
 
 
 def first_line(doc):
@@ -22,12 +23,28 @@ def first_line(doc):
     return ''
 
 
+class DelayedLoader(object):
+    """contains info for delayed creation of tasks from a task-creator
+
+    @ivar creator: reference to task-creator function
+    @ivar task_dep: (str) name of task that should be executed before the
+                    the loader call the creator function
+    """
+    def __init__(self, creator, executed=None):
+        self.creator = creator
+        self.task_dep = executed
+
+# used to indicate that a task had DelayedLoader but was already created
+DelayedLoaded = False
+
+
 class Task(object):
     """Task
 
     @ivar name string
     @ivar actions: list - L{BaseAction}
     @ivar clean_actions: list - L{BaseAction}
+    @ivar loader (DelayedLoader)
     @ivar teardown (list - L{BaseAction})
     @ivar targets: (list -string)
     @ivar task_dep: (list - string)
@@ -51,6 +68,9 @@ class Task(object):
 
     @ivar options: (dict) calculated params values (from getargs and taskopt)
     @ivar taskopt: (cmdparse.CmdParse)
+    @ivar pos_arg: (str) name of parameter in action to receive positional
+                     parameters from command line
+    @ivar pos_arg_val: (list - str) list of positional parameters values
     @ivar custom_title: function reference that takes a task object as
                         parameter and returns a string.
     """
@@ -71,19 +91,21 @@ class Task(object):
                   'teardown': ((list, tuple), ()),
                   'doc': (string_types, (None,)),
                   'params': ((list, tuple,), ()),
-                  'verbosity': ((), (None,0,1,2,)),
+                  'pos_arg': (string_types, (None,)),
+                  'verbosity': ((), (None, 0, 1, 2,)),
                   'getargs': ((dict,), ()),
                   'title': ((types.FunctionType,), (None,)),
                   'watch': ((list, tuple), ()),
-                  }
+    }
 
 
     def __init__(self, name, actions, file_dep=(), targets=(),
                  task_dep=(), uptodate=(),
                  calc_dep=(), setup=(), clean=(), teardown=(),
                  is_subtask=False, has_subtask=False,
-                 doc=None, params=(), verbosity=None, title=None, getargs=None,
-                 watch=()):
+                 doc=None, params=(), pos_arg=None,
+                 verbosity=None, title=None, getargs=None,
+                 watch=(), loader=None):
         """sanity checks and initialization
 
         @param params: (list of dict for parameters) see cmdparse.CmdOption
@@ -102,6 +124,8 @@ class Task(object):
         self.check_attr(name, 'teardown', teardown, self.valid_attr['teardown'])
         self.check_attr(name, 'doc', doc, self.valid_attr['doc'])
         self.check_attr(name, 'params', params, self.valid_attr['params'])
+        self.check_attr(name, 'pos_arg', pos_arg,
+                        self.valid_attr['pos_arg'])
         self.check_attr(name, 'verbosity', verbosity,
                         self.valid_attr['verbosity'])
         self.check_attr(name, 'getargs', getargs, self.valid_attr['getargs'])
@@ -110,7 +134,10 @@ class Task(object):
 
         self.name = name
         self.taskcmd = TaskParse([CmdOption(opt) for opt in params])
+        self.params = params # save just for use on command `info`
         self.options = self._init_options()
+        self.pos_arg = pos_arg
+        self.pos_arg_val = None # to be set when parsing command line
         self.setup_tasks = list(setup)
 
         # actions
@@ -122,12 +149,20 @@ class Task(object):
 
         self._init_deps(file_dep, task_dep, calc_dep)
 
-        self.value_savers = []
-        self.uptodate = self._init_uptodate(uptodate) if uptodate else []
+        # loaders create an implicity task_dep
+        self.loader = loader
+        if self.loader and self.loader.task_dep:
+            self.task_dep.append(loader.task_dep)
+
+        uptodate = uptodate if uptodate else []
 
         self.getargs = getargs
         if self.getargs:
-            self._init_getargs()
+            uptodate.extend(self._init_getargs())
+
+        self.value_savers = []
+        self.uptodate = self._init_uptodate(uptodate)
+
         self.targets = targets
         self.is_subtask = is_subtask
         self.has_subtask = has_subtask
@@ -184,8 +219,8 @@ class Task(object):
                 uptodate.append((item, [], {}))
             elif isinstance(item, tuple):
                 call = item[0]
-                args = list(item[1]) if len(item)>1 else []
-                kwargs = item[2] if len(item)>2 else {}
+                args = list(item[1]) if len(item) > 1 else []
+                kwargs = item[2] if len(item) > 2 else {}
                 uptodate.append((call, args, kwargs))
             elif isinstance(item, six.string_types):
                 uptodate.append((item, [], {}))
@@ -228,11 +263,12 @@ class Task(object):
 
 
     # FIXME should support setup also
-    _expand_map = {'task_dep': _expand_task_dep,
-                   'file_dep': _expand_file_dep,
-                   'calc_dep': _expand_calc_dep,
-                   'uptodate': _extend_uptodate,
-                   }
+    _expand_map = {
+        'task_dep': _expand_task_dep,
+        'file_dep': _expand_file_dep,
+        'calc_dep': _expand_calc_dep,
+        'uptodate': _extend_uptodate,
+    }
     def update_deps(self, deps):
         """expand all kinds of dep input"""
         for dep, dep_values in six.iteritems(deps):
@@ -251,6 +287,8 @@ class Task(object):
 
     def _init_getargs(self):
         """task getargs attribute define implicit task dependencies"""
+        check_result = set()
+
         for arg_name, desc in six.iteritems(self.getargs):
 
             # tuple (task_id, key_name)
@@ -263,7 +301,9 @@ class Task(object):
                 raise InvalidTask(msg)
 
             if parts[0] not in self.setup_tasks:
-                self.setup_tasks.append(parts[0])
+                check_result.add(parts[0])
+
+        return [result_dep(t) for t in check_result]
 
 
     @staticmethod
@@ -299,7 +339,8 @@ class Task(object):
     def actions(self):
         """lazy creation of action instances"""
         if self._action_instances is None:
-            self._action_instances = [create_action(a, self) for a in self._actions]
+            self._action_instances = [
+                create_action(a, self) for a in self._actions]
         return self._action_instances
 
 
@@ -387,7 +428,7 @@ class Task(object):
 
 
     # when using multiprocessing Tasks are pickled.
-    def __getstate__(self):
+    def pickle_safe_dict(self):
         """remove attributes that might contain unpickleble content
         mostly probably closures
         """
@@ -401,12 +442,12 @@ class Task(object):
         del to_pickle['uptodate']
         return to_pickle
 
-    def __eq__(self, other):
-        return self.name == other.name
-
     def update_from_pickle(self, pickle_obj):
         """update self with data from pickled Task"""
-        self.__dict__.update(pickle_obj.__dict__)
+        self.__dict__.update(pickle_obj)
+
+    def __eq__(self, other):
+        return self.name == other.name
 
     def __lt__(self, other):
         """used on default sorting of tasks (alphabetically by name)"""
@@ -426,7 +467,7 @@ def dict_to_task(task_dict):
     # check required fields
     if 'actions' not in task_dict:
         raise InvalidTask("Task %s must contain 'actions' field. %s" %
-                          (task_dict['name'],task_dict))
+                          (task_dict['name'], task_dict))
 
     # user friendly. dont go ahead with invalid input.
     task_attrs = list(six.iterkeys(task_dict))
@@ -434,7 +475,7 @@ def dict_to_task(task_dict):
     for key in task_attrs:
         if key not in valid_attrs:
             raise InvalidTask("Task %s contains invalid field: '%s'"%
-                              (task_dict['name'],key))
+                              (task_dict['name'], key))
 
     return Task(**task_dict)
 
@@ -461,3 +502,46 @@ def clean_targets(task, dryrun):
             six.print_(msg % (task.name, dir_))
             if not dryrun:
                 os.rmdir(dir_)
+
+
+# uptodate
+class result_dep(UptodateCalculator):
+    """check if result of the given task was modified
+    """
+    def __init__(self, dep_task_name):
+        self.dep_name = dep_task_name
+        self.result_name = '_result:%s' % self.dep_name
+
+    def configure_task(self, task):
+        """to be called by doit when create the task"""
+        # result_dep creates an implicit task_dep
+        task.setup_tasks.append(self.dep_name)
+
+    def _result_single(self):
+        """get result from a single task"""
+        return self.get_val(self.dep_name, 'result:')
+
+    def _result_group(self, dep_task):
+        """get result from a group task
+        the result is the combination of results of all sub-tasks
+        """
+        prefix = dep_task.name + ":"
+        sub_tasks = {}
+        for sub in dep_task.task_dep:
+            if sub.startswith(prefix):
+                sub_tasks[sub] = self.get_val(sub, 'result:')
+        return sub_tasks
+
+    def __call__(self, task, values):
+        """return True if result is the same as last run"""
+        dep_task = self.tasks_dict[self.dep_name]
+        if not dep_task.has_subtask:
+            dep_result = self._result_single()
+        else:
+            dep_result = self._result_group(dep_task)
+        task.value_savers.append(lambda: {self.result_name: dep_result})
+
+        last_success = values.get(self.result_name)
+        if last_success is None:
+            return False
+        return last_success == dep_result
